# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA"

ANALOG_CHANNEL1 = 1
ANALOG_CHANNEL2 = 2

ADC_SAMPLING_TIME_9NS = 9
ADC_SAMPLING_TIME_16NS = 16
ADC_SAMPLING_TIME_53NS = 53
ADC_SAMPLING_TIME_103NS = 103
ADC_SAMPLING_TIME_222NS = 222
ADC_SAMPLING_TIME_403NS = 403
ADC_SAMPLING_TIME_2422NS = 2422
ADC_SAMPLING_TIME_5066NS = 5066

ADC_SAMPLING_TIMES = [ADC_SAMPLING_TIME_9NS, ADC_SAMPLING_TIME_16NS, ADC_SAMPLING_TIME_53NS,
                      ADC_SAMPLING_TIME_103NS, ADC_SAMPLING_TIME_222NS, ADC_SAMPLING_TIME_403NS,
                      ADC_SAMPLING_TIME_2422NS, ADC_SAMPLING_TIME_5066NS]

ADC_SAMPLING_TIME_DICT = {k: v for k, v in enumerate(ADC_SAMPLING_TIMES)}
ADC_SAMPLING_TIME_DICT_INV = {v: k for k, v in ADC_SAMPLING_TIME_DICT.items()}


ADC_SAMPLING_TIMES = [9, 16, 53, 103, 222, 403, 2422, 5066]
